#!/usr/bin/env python
import os
import sys
import grp
import pwd
import argparse
from shutil import copyfile

def get_framework_certificate_path(app_root_path, framework_name):
    return os.path.join(app_root_path, "Contents/Frameworks/" + framework_name + ".framework/Resources/LightSpeed CA" + ".pem")

def get_certificate_path(app_root_path):
    fullPath = get_framework_certificate_path(app_root_path, "XSModelCore")
    if not os.path.exists(fullPath):
        # Older versions of Lightspeed only had XSModel, not XSModelCore
        fullPath = get_framework_certificate_path(app_root_path, "XSModel")
        if not os.path.exists(fullPath):
            # In even older versions of Lightspeed, everything was in BlackIce
            fullPath = get_framework_certificate_path(app_root_path, "BlackIce")
    return fullPath

def is_valid_application_bundle(parser, arg):
    """
        Determines if a path really represents a Lightspeed application bundle

        Returns the full path to the certificate we're trying to replace.
    """

    applicationCertPath = get_certificate_path(arg)
    if not os.path.exists(applicationCertPath):
        sys.exit("The application at path '%s' cannot be fixed with this tool." % arg)
    else:
        return applicationCertPath

def is_valid_certificate_path(parser, arg):
    """
        Validates if there is a file at the certificate path

        Returns the path if it exists.
    """
    if not os.path.exists(arg):
        sys.exit("There is no certificate at path '%s'." % arg)
    else:
        return arg

def update_application(certificate_path, application_framework_path):
    """
        Update the certificate in the application
    """
    try:
        # Save the user and group of the file we're about to replace
        stat_info = os.stat(application_framework_path)
        uid = stat_info.st_uid
        gid = stat_info.st_gid
        # Copy the new certificate
        copyfile(certificate_path, application_framework_path)
        # Restore user and group to previous values
        os.chown(application_framework_path, uid, gid)
    except IOError:
        sys.exit("Unable to copy certificate to destination '%s'." % application_framework_path)

# Start of script

if os.geteuid() != 0:
    sys.exit("This program must be run as root. Use sudo.")

parser = argparse.ArgumentParser(description = 'Update Lightspeed client certificate.')

parser.add_argument("--certificate", dest="certpath", required=True,
                    help="path to the updated certificate", metavar="certificate_path",
                    type=lambda x: is_valid_certificate_path(parser, x))
parser.add_argument("--bundle", dest="destfilepath", required=True,
                    help="path to the Lightspeed application to update", metavar="application_path",
                    type=lambda x: is_valid_application_bundle(parser, x))

args = parser.parse_args()

update_application(args.certpath, args.destfilepath)
